#include "sandbox/sandbox_poc/pocdll/exports.h"
#include "sandbox/sandbox_poc/pocdll/utils.h"
#include <Objbase.h>
#include <Exdisp.h>

//#pragma section(".custom_section",read,write)
//volatile __declspec(allocate(".custom_section")) char *strMsg = "Message to be Sent";

void TrySetClipboard(FILE *output)
{
	char *buffer;
	HGLOBAL clipbuffer;
	HWND hwndMain = GetForegroundWindow();
	HANDLE retHandle = NULL;
	if(OpenClipboard(hwndMain))
	{
		EmptyClipboard();
		clipbuffer = GlobalAlloc(GMEM_DDESHARE, 0x20);
		buffer = (char*)GlobalLock(clipbuffer);
		strcpy(buffer, LPCSTR("Test Clipboard Data"));
		GlobalUnlock(clipbuffer);
		retHandle = SetClipboardData(CF_TEXT, clipbuffer);
		if(retHandle == NULL) 
		{
			fprintf(output, "[BLOCKED] {CF_TEXT} SetClipboardData() Error: %X\r\n", GetLastError());
		}
		else
		{
			fprintf(output, "[GRANTED] SetClipboardData()\r\n");
		}
		CloseClipboard();
	}
	else
	{
		fprintf(output, "[BLOCKED] {SET} OpenClipboard() Error: %X\r\n", GetLastError());
	}
}

void TryReadClipboard(FILE *output)
{
	HANDLE hClipData;
	HWND hwndMain = GetForegroundWindow();
	HANDLE retHandle = NULL;
	char *buffer;

	if(OpenClipboard(hwndMain))
	{
		hClipData = GetClipboardData(CF_TEXT);
		if(hClipData == NULL) 
		{
			fprintf(output, "[BLOCKED] {CF_TEXT} GetClipboardData() Error: %X\r\n", GetLastError());
		}
		else
		{
			fprintf(output, "[GRANTED] GetClipboardData()\r\n");
		}
		buffer = (char*)GlobalLock(hClipData);
		GlobalUnlock(hClipData);
		CloseClipboard();
	}
	else
	{
		fprintf(output, "[BLOCKED] {GET} OpenClipboard() Error: %X\r\n", GetLastError());
	}

}

void TryCreateDesktop(FILE *output)
{
	HDESK new_desktop = NULL;
	new_desktop = CreateDesktop(TEXT("ChrisDesktop"), NULL, NULL, 0x0, DESKTOP_CREATEWINDOW, NULL);
	if(new_desktop == NULL)
		fprintf(output, "[BLOCKED] {DESKTOP_CREATEWINDOW} CreateDesktop() Error: %X\r\n", GetLastError());
	else
	{
		fprintf(output, "[GRANTED] {DESKTOP_CREATEWINDOW} CreateDesktop()\r\n");
		CloseDesktop(new_desktop);
	}
}

void TryOpenDesktop(FILE *output)
{
	HDESK new_desktop = NULL;
	HWINSTA hwinsta = NULL;

	hwinsta = OpenWindowStation(TEXT("winsta0"), FALSE, WINSTA_CREATEDESKTOP);
	if(hwinsta == NULL)
	{
		fprintf(output, "[BLOCKED] {winsta0} {WINSTA_CREATEDESKTOP} OpenWindowStation() Error: %X\r\n", GetLastError());
	}
	else
	{
		fprintf(output, "[GRANTED] {Winsta0} {WINSTA_CREATEDESKTOP} OpenWindowStation()\r\n");
		if(!SetProcessWindowStation(hwinsta))
		{
			fprintf(output, "[BLOCKED] {winsta0} SetProcessWindowStation() Error: %X\r\n", GetLastError());
		}
		else
		{
			fprintf(output, "[GRANTED] {winsta0} SetProcessWindowStation()\r\n");
		}
	}

	new_desktop = OpenDesktop(TEXT("Default"), 0, FALSE, DESKTOP_CREATEWINDOW);
	if(new_desktop == NULL)
	{
		fprintf(output, "[BLOCKED] {Default} {DESKTOP_CREATEWINDOW} OpenDesktop() Error: %X\r\n", GetLastError());
	}
	else
	{
		fprintf(output, "[GRANTED] {Default} {DESKTOP_CREATEWINDOW} OpenDesktop()\r\n");
	}

	if(new_desktop)
		CloseDesktop(new_desktop);

	if(hwinsta)
		CloseWindowStation(hwinsta);
}

void ReadSystemParametersInfo(FILE *output)
{
	int aMouseInfo[3];
	BOOL fResult;

	SetLastError(0);

	fResult = SystemParametersInfoA(SPI_GETMOUSE, 0, &aMouseInfo, 0);
	if(fResult == 0)
		fprintf(output, "[BLOCKED] {SPI_GETMOUSE} Get SystemParametersInfo() Error: %X\r\n", GetLastError());
	else
		fprintf(output, "[GRANTED] {SPI_GETMOUSE} Get SystemParametersInfo()\r\n");
}

void SetSystemParametersInfo(FILE *output)
{
	int aMouseInfo[3];
	BOOL fResult;
	
	SetLastError(0);
	fResult = SystemParametersInfoA(SPI_SETMOUSE, 0, &aMouseInfo, SPIF_SENDCHANGE);
	if(fResult == 0)
		fprintf(output, "[BLOCKED] {SPI_SETMOUSE} Set SystemParametersInfo() Error: %X\r\n", GetLastError());
	else
		fprintf(output, "[GRANTED] {SPI_SETMOUSE} Set SystemParametersInfclso()\r\n");
}

void MsgDoIt()
{
	fprintf(stderr, "Send Message");
}

void TryWindowsMessageBroadcast(FILE *output)
{
	BOOL ret_val;
	DWORD	addr;
	addr = (DWORD)MsgDoIt;

	ret_val = SendMessageA(HWND_BROADCAST, WM_TIMER, 0, addr);
	if(ret_val == FALSE)
		fprintf(output, "[BLOCKED] {WM_TIMER} SendMessage() Broadcast Error: %X\r\n", GetLastError());
	else
		fprintf(output, "[GRANTED] {WM_TIMER} SendMessage() Broadcast\r\n");
}

void TrySetWindowsHook(FILE *output)
{
	typedef int (__stdcall *MYPROC)(HWND a, LPCSTR b, LPCSTR c, INT d); 
	HINSTANCE hinstLib;
	static HHOOK hhookSysMsg;
	HOOKPROC hkprcSysMsg;

	hinstLib = LoadLibrary(TEXT("User32"));
	if(hinstLib != NULL)
	{
		hkprcSysMsg = (HOOKPROC)GetProcAddress(hinstLib, "MessageBoxA");
		if(hkprcSysMsg != NULL)
		{
			hhookSysMsg = SetWindowsHookEx(WH_KEYBOARD, hkprcSysMsg, hinstLib, 0);
			if(hhookSysMsg != NULL)
			{
				fprintf(output, "[GRANTED] {WH_KEYBOARD} SetWindowsHookEx()\r\n");
				UnhookWindowsHookEx(hhookSysMsg);
			}
			else
			{
				fprintf(output, "[BLOCKED] {WH_KEYBOARD} SetWindowsHookEx() Error: %X\r\n", GetLastError());
			}
		}
		else
		{
			fprintf(output, "[ERROR] {SetWindowsHookEx} GetProcessAddress() %X\r\n", GetLastError());
		}
	}
	else
	{
		fprintf(output, "[BLOCKED] {SetWindowsHookEx} LoadLibrary() %X\r\n", GetLastError());
	}

	if(hinstLib)
		FreeLibrary(hinstLib);
}

void TryCOMInstantiation(FILE *output)
{
	IWebBrowser2*    pBrowser2;
	HRESULT hr;

	hr = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
	if(FAILED(hr))
	{
		fprintf(output, "[ERROR] {COINIT_APARTMENTTHREADED} CoInitializeEx() Error: %X\r\n", hr);
		return;
	}

	hr = CoCreateInstance(CLSID_InternetExplorer, NULL, CLSCTX_LOCAL_SERVER, IID_IWebBrowser2, (void**)&pBrowser2);
	if(FAILED(hr))
	{
		if(hr = CO_E_SERVER_EXEC_FAILURE)
		{
			fprintf(output, "[BLOCKED] {IID_IWebBrowser2} CoCreateInstance() Error: %X\r\n", hr);
		}
		else
		{
			fprintf(output, "[ERROR] {IID_IWebBrowser2} CoCreateInstance() Error: %X\r\n", hr);
		}
	}
	else
	{
		fprintf(output, "[GRANTED] CoCreateInstance()\r\n");
	}

	if(pBrowser2 != NULL)
		pBrowser2->Release();

	CoUninitialize();

}

void TryNamedPipes(FILE *output, char *strdesiredAccess, DWORD desiredAccess)
{
	WIN32_FIND_DATAW FindNamedPipes;
	WCHAR pipe[MAX_PATH];
	HANDLE hFind;
	HANDLE hNP;
	WCHAR *known_pipes[] = {L"\\\\.\\pipe\\lsass", L"\\\\.\\pipe\\ntsvcs", L"\\\\.\\pipe\\scerpc",
							L"\\\\.\\pipe\\protected_storage", L"\\\\.\\pipe\\plugplay", L"\\\\.\\pipe\\epmapper",
							L"\\\\.\\pipe\\eventlog", L"\\\\.\\pipe\\atsvc", L"\\\\.\\pipe\\wkssvc",
							L"\\\\.\\pipe\\keysvc", L"\\\\.\\pipe\\trkwks", L"\\\\.\\pipe\\srvsvc"};

	int num_of_pipes = sizeof(known_pipes) / sizeof(WCHAR*);
	int i = 0;

	hFind = FindFirstFileW(L"\\\\.\\pipe\\*", &FindNamedPipes);
	if(hFind == INVALID_HANDLE_VALUE)
	{
		fprintf(output, "[ERROR] {%s} FindFirstFile Error: %X\r\n", strdesiredAccess, GetLastError());
		fprintf(output, "[INFO] Using named pipe heuristics\r\n");

		for(i = 0; i < num_of_pipes; i++)
		{
			hNP = CreateFile(known_pipes[i], desiredAccess, 0, NULL, OPEN_EXISTING, 0, NULL);
			if(hNP == INVALID_HANDLE_VALUE)
			{
				fprintf(output, "[BLOCKED] {%s} %ls Error: %X\r\n", strdesiredAccess, known_pipes[i], GetLastError());
			}
			else
			{
				fprintf(output, "[GRANTED] {%s} %ls\r\n", strdesiredAccess, known_pipes[i]);
				CloseHandle(hNP);
			}
		}
		return;
	}

	memset(pipe, 0x0, sizeof(pipe));
	wcscat(pipe, L"\\\\.\\pipe\\");
	wcscat_s(pipe, MAX_PATH - 10, FindNamedPipes.cFileName);
	hNP = CreateFile(pipe, desiredAccess, 0, NULL, OPEN_EXISTING, 0, NULL);
	if(hNP == INVALID_HANDLE_VALUE)
	{
		fprintf(output, "[BLOCKED] {%s} %ls Error: %X\r\n", strdesiredAccess, pipe, GetLastError());
	}
	else
	{
		fprintf(output, "[GRANTED] {%s} %ls\r\n", strdesiredAccess, pipe);
		CloseHandle(hNP);
	}

	BOOL hasMore = FindNextFile(hFind, &FindNamedPipes);
	while(hasMore)
	{
		memset(pipe, 0x0, sizeof(pipe));
		wcscat(pipe, L"\\\\.\\pipe\\");
		wcscat_s(pipe, MAX_PATH - 10, FindNamedPipes.cFileName);

		while(1)
		{
			hNP = CreateFile(pipe, desiredAccess, 0, NULL, OPEN_EXISTING, 0, NULL);
			if(hNP == INVALID_HANDLE_VALUE)
			{
				if(GetLastError() != ERROR_PIPE_BUSY)
				{
					fprintf(output, "[BLOCKED] {%s} %ls Error: %X\r\n", strdesiredAccess, pipe, GetLastError());
					break;
				}
				if(!WaitNamedPipe(pipe, 20000))
				{
					fprintf(output, "[ERROR] {%s} %ls Error: %s\r\n", strdesiredAccess, pipe,"Pipe Timeout");
					break;
				}
			}
			else
			{
				fprintf(output, "[GRANTED] {%s} %ls\r\n", strdesiredAccess, pipe);
				CloseHandle(hNP);
				break;
			}
		}
		hasMore = FindNextFile(hFind, &FindNamedPipes);
	}
	FindClose(hFind);

}

void POCDLL_API TestChrisTests(HANDLE log) 
{
	HandleToFile handle2file;
	FILE *output = handle2file.Translate(log, "w");
	
	TrySetClipboard(output);
	TryReadClipboard(output);
	ReadSystemParametersInfo(output);
	SetSystemParametersInfo(output);
	TryWindowsMessageBroadcast(output);
	TrySetWindowsHook(output);
	//TryCOMInstantiation(output);

	//Try all desired access types
	//TryNamedPipes(output, "PIPE_ACCESS_DUPLEX", PIPE_ACCESS_DUPLEX);
	TryNamedPipes(output, "PIPE_ACCESS_INBOUND", PIPE_ACCESS_INBOUND);
	TryNamedPipes(output, "PIPE_ACCESS_OUTBOUND", PIPE_ACCESS_OUTBOUND);

	TryCreateDesktop(output);
	TryOpenDesktop(output);

}