// Copyright (c) 2006-2008 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include "sandbox/sandbox_poc/pocdll/exports.h"
#include "sandbox/sandbox_poc/pocdll/utils.h"

// This file contains the tests used to verify the security of the file system.

// Tries to open a file and outputs the result.
// "path" can contain environment variables.
// "output" is the stream for the logging.
void TryOpenFile(wchar_t *path, FILE *output, char *strDesiredAccess, DWORD desiredAccess) {
  wchar_t path_expanded[MAX_PATH] = {0};
  DWORD size = ::ExpandEnvironmentStrings(path, path_expanded, MAX_PATH - 1);
  if (!size) {
    fprintf(output, "[ERROR] Cannot expand \"%S\". Error %S.\r\n", path,
            ::GetLastError());
  }

  //http://msdn.microsoft.com/en-us/library/aa363858%28VS.85%29.aspx
  //dwDesiredAccess
  //If this parameter is zero, the application can query certain metadata such as 
  //file, directory, or device attributes without accessing that file or device, 
  //even if GENERIC_READ access would have been denied.
  HANDLE file;
  file = ::CreateFile(path_expanded,
                      desiredAccess,
                      FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
                      NULL,  // No security attributes
                      OPEN_EXISTING,
                      FILE_FLAG_BACKUP_SEMANTICS,
                      NULL);  // No template

  if (file && INVALID_HANDLE_VALUE != file) {
	  fprintf(output, "[GRANTED] {%s} Opening file \"%S\". Handle 0x%p\r\n", strDesiredAccess, path,
            file);
    ::CloseHandle(file);
  } else {
	  fprintf(output, "[BLOCKED] {%s} Opening file \"%S\". Error %d.\r\n", strDesiredAccess, path,
            ::GetLastError());
  }
}

void TryWriteFile(wchar_t *path, FILE *output) {
  wchar_t path_expanded[MAX_PATH] = {0};
  DWORD size = ::ExpandEnvironmentStrings(path, path_expanded, MAX_PATH - 1);
  if (!size) {
	  fprintf(output, "[ERROR] {TryWriteFile} Cannot expand \"%S\". Error %S.\r\n", path,
            ::GetLastError());
  }

  HANDLE file;
  wcscat_s(path_expanded, MAX_PATH - wcslen(path_expanded), L"\\test.txt");
  file = ::CreateFile(path_expanded, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

  if (file != INVALID_HANDLE_VALUE) 
  {
	  fprintf(output, "[GRANTED] {GENERIC_WRITE} Writing file \"%S\". Handle 0x%p\r\n", path,file);
    ::CloseHandle(file);
	::DeleteFile(path_expanded);
  } 
  else 
  {
	  fprintf(output, "[BLOCKED] {GENERIC_WRITE} Writing file \"%S\". Error %d.\r\n", path, ::GetLastError());
  }
}

void POCDLL_API TestFileSystem(HANDLE log) {
  HandleToFile handle2file;
  FILE *output = handle2file.Translate(log, "w");

  //CV
  //%SystemDrive%
  TryOpenFile(L"%SystemDrive%", output, "ZERO", 0x0);
  TryOpenFile(L"%SystemDrive%", output, "GENERIC_READ", GENERIC_READ);
  TryOpenFile(L"%SystemDrive%", output, "GENERIC_WRITE", GENERIC_WRITE);
  TryOpenFile(L"%SystemDrive%", output, "FILE_ADD_FILE", FILE_ADD_FILE);
  TryOpenFile(L"%SystemDrive%", output, "FILE_ADD_SUBDIRECTORY", FILE_ADD_SUBDIRECTORY);
  TryOpenFile(L"%SystemDrive%", output, "FILE_APPEND_DATA", FILE_APPEND_DATA);
  TryOpenFile(L"%SystemDrive%", output, "FILE_CREATE_PIPE_INSTANCE", FILE_CREATE_PIPE_INSTANCE);
  TryOpenFile(L"%SystemDrive%", output, "FILE_DELETE_CHILD", FILE_DELETE_CHILD);
  TryOpenFile(L"%SystemDrive%", output, "FILE_LIST_DIRECTORY", FILE_LIST_DIRECTORY);
  TryOpenFile(L"%SystemDrive%", output, "FILE_READ_ATTRIBUTES", FILE_READ_ATTRIBUTES);
  TryOpenFile(L"%SystemDrive%", output, "FILE_READ_DATA", FILE_READ_DATA);
  TryOpenFile(L"%SystemDrive%", output, "FILE_READ_EA", FILE_READ_EA);
  TryOpenFile(L"%SystemDrive%", output, "FILE_TRAVERSE", FILE_TRAVERSE);
  TryOpenFile(L"%SystemDrive%", output, "FILE_WRITE_ATTRIBUTES", FILE_WRITE_ATTRIBUTES);
  TryOpenFile(L"%SystemDrive%", output, "FILE_WRITE_DATA", FILE_WRITE_DATA);
  TryOpenFile(L"%SystemDrive%", output, "FILE_WRITE_EA", FILE_WRITE_EA);  
  TryOpenFile(L"%SystemDrive%", output, "WRITE_DAC", WRITE_DAC);

  //%SystemRoot%
  TryOpenFile(L"%SystemRoot%", output, "ZERO", 0x0);
  TryOpenFile(L"%SystemRoot%", output, "GENERIC_READ", GENERIC_READ);
  TryOpenFile(L"%SystemRoot%", output, "GENERIC_WRITE", GENERIC_WRITE);
  TryOpenFile(L"%SystemRoot%", output, "FILE_ADD_FILE", FILE_ADD_FILE);
  TryOpenFile(L"%SystemRoot%", output, "FILE_ADD_SUBDIRECTORY", FILE_ADD_SUBDIRECTORY);
  TryOpenFile(L"%SystemRoot%", output, "FILE_APPEND_DATA", FILE_APPEND_DATA);
  TryOpenFile(L"%SystemRoot%", output, "FILE_CREATE_PIPE_INSTANCE", FILE_CREATE_PIPE_INSTANCE);
  TryOpenFile(L"%SystemRoot%", output, "FILE_DELETE_CHILD", FILE_DELETE_CHILD);
  TryOpenFile(L"%SystemRoot%", output, "FILE_LIST_DIRECTORY", FILE_LIST_DIRECTORY);
  TryOpenFile(L"%SystemRoot%", output, "FILE_READ_ATTRIBUTES", FILE_READ_ATTRIBUTES);
  TryOpenFile(L"%SystemRoot%", output, "FILE_READ_DATA", FILE_READ_DATA);
  TryOpenFile(L"%SystemRoot%", output, "FILE_READ_EA", FILE_READ_EA);
  TryOpenFile(L"%SystemRoot%", output, "FILE_TRAVERSE", FILE_TRAVERSE);
  TryOpenFile(L"%SystemRoot%", output, "FILE_WRITE_ATTRIBUTES", FILE_WRITE_ATTRIBUTES);
  TryOpenFile(L"%SystemRoot%", output, "FILE_WRITE_DATA", FILE_WRITE_DATA);
  TryOpenFile(L"%SystemRoot%", output, "FILE_WRITE_EA", FILE_WRITE_EA); 
  TryOpenFile(L"%SystemRoot%", output, "WRITE_DAC", WRITE_DAC);

  //%ProgramFiles%
  TryOpenFile(L"%ProgramFiles%", output, "ZERO", 0x0);
  TryOpenFile(L"%ProgramFiles%", output, "GENERIC_READ", GENERIC_READ);
  TryOpenFile(L"%ProgramFiles%", output, "GENERIC_WRITE", GENERIC_WRITE);
  TryOpenFile(L"%ProgramFiles%", output, "FILE_ADD_FILE", FILE_ADD_FILE);
  TryOpenFile(L"%ProgramFiles%", output, "FILE_ADD_SUBDIRECTORY", FILE_ADD_SUBDIRECTORY);
  TryOpenFile(L"%ProgramFiles%", output, "FILE_APPEND_DATA", FILE_APPEND_DATA);
  TryOpenFile(L"%ProgramFiles%", output, "FILE_CREATE_PIPE_INSTANCE", FILE_CREATE_PIPE_INSTANCE);
  TryOpenFile(L"%ProgramFiles%", output, "FILE_DELETE_CHILD", FILE_DELETE_CHILD);
  TryOpenFile(L"%ProgramFiles%", output, "FILE_LIST_DIRECTORY", FILE_LIST_DIRECTORY);
  TryOpenFile(L"%ProgramFiles%", output, "FILE_READ_ATTRIBUTES", FILE_READ_ATTRIBUTES);
  TryOpenFile(L"%ProgramFiles%", output, "FILE_READ_DATA", FILE_READ_DATA);
  TryOpenFile(L"%ProgramFiles%", output, "FILE_READ_EA", FILE_READ_EA);
  TryOpenFile(L"%ProgramFiles%", output, "FILE_TRAVERSE", FILE_TRAVERSE);
  TryOpenFile(L"%ProgramFiles%", output, "FILE_WRITE_ATTRIBUTES", FILE_WRITE_ATTRIBUTES);
  TryOpenFile(L"%ProgramFiles%", output, "FILE_WRITE_DATA", FILE_WRITE_DATA);
  TryOpenFile(L"%ProgramFiles%", output, "FILE_WRITE_EA", FILE_WRITE_EA); 
  TryOpenFile(L"%ProgramFiles%", output, "WRITE_DAC", WRITE_DAC); 

  //%SystemRoot%\\System32
  TryOpenFile(L"%SystemRoot%\\System32", output, "ZERO", 0x0);
  TryOpenFile(L"%SystemRoot%\\System32", output, "GENERIC_READ", GENERIC_READ);
  TryOpenFile(L"%SystemRoot%\\System32", output, "GENERIC_WRITE", GENERIC_WRITE);
  TryOpenFile(L"%SystemRoot%\\System32", output, "FILE_ADD_FILE", FILE_ADD_FILE);
  TryOpenFile(L"%SystemRoot%\\System32", output, "FILE_ADD_SUBDIRECTORY", FILE_ADD_SUBDIRECTORY);
  TryOpenFile(L"%SystemRoot%\\System32", output, "FILE_APPEND_DATA", FILE_APPEND_DATA);
  TryOpenFile(L"%SystemRoot%\\System32", output, "FILE_CREATE_PIPE_INSTANCE", FILE_CREATE_PIPE_INSTANCE);
  TryOpenFile(L"%SystemRoot%\\System32", output, "FILE_DELETE_CHILD", FILE_DELETE_CHILD);
  TryOpenFile(L"%SystemRoot%\\System32", output, "FILE_LIST_DIRECTORY", FILE_LIST_DIRECTORY);
  TryOpenFile(L"%SystemRoot%\\System32", output, "FILE_READ_ATTRIBUTES", FILE_READ_ATTRIBUTES);
  TryOpenFile(L"%SystemRoot%\\System32", output, "FILE_READ_DATA", FILE_READ_DATA);
  TryOpenFile(L"%SystemRoot%\\System32", output, "FILE_READ_EA", FILE_READ_EA);
  TryOpenFile(L"%SystemRoot%\\System32", output, "FILE_TRAVERSE", FILE_TRAVERSE);
  TryOpenFile(L"%SystemRoot%\\System32", output, "FILE_WRITE_ATTRIBUTES", FILE_WRITE_ATTRIBUTES);
  TryOpenFile(L"%SystemRoot%\\System32", output, "FILE_WRITE_DATA", FILE_WRITE_DATA);
  TryOpenFile(L"%SystemRoot%\\System32", output, "FILE_WRITE_EA", FILE_WRITE_EA);
  TryOpenFile(L"%SystemRoot%\\System32", output, "WRITE_DAC", WRITE_DAC);

  //%SystemRoot%\\explorer.exe
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "ZERO", 0x0);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "GENERIC_READ", GENERIC_READ);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "GENERIC_WRITE", GENERIC_WRITE);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "GENERIC_EXECUTE", GENERIC_EXECUTE);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "FILE_EXECUTE", FILE_EXECUTE);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "FILE_READ_ATTRIBUTES", FILE_READ_ATTRIBUTES);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "STANDARD_RIGHTS_EXECUTE", STANDARD_RIGHTS_EXECUTE);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "SYNCHRONIZE", SYNCHRONIZE);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "FILE_READ_DATA", FILE_READ_DATA);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "FILE_READ_EA", FILE_READ_EA);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "STANDARD_RIGHTS_READ", STANDARD_RIGHTS_READ);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "FILE_APPEND_DATA", FILE_APPEND_DATA);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "FILE_WRITE_ATTRIBUTES", FILE_WRITE_ATTRIBUTES);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "FILE_WRITE_DATA", FILE_WRITE_DATA);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "FILE_WRITE_EA", FILE_WRITE_EA);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "STANDARD_RIGHTS_WRITE", STANDARD_RIGHTS_WRITE);
  TryOpenFile(L"%SystemRoot%\\explorer.exe", output, "WRITE_DAC", WRITE_DAC);

  //%SystemRoot%\\Cursors\\arrow_i.cur
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "ZERO", 0x0);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "GENERIC_READ", GENERIC_READ);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "GENERIC_WRITE", GENERIC_WRITE);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "GENERIC_EXECUTE", GENERIC_EXECUTE);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "FILE_EXECUTE", FILE_EXECUTE);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "FILE_READ_ATTRIBUTES", FILE_READ_ATTRIBUTES);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "STANDARD_RIGHTS_EXECUTE", STANDARD_RIGHTS_EXECUTE);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "SYNCHRONIZE", SYNCHRONIZE);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "FILE_READ_DATA", FILE_READ_DATA);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "FILE_READ_EA", FILE_READ_EA);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "STANDARD_RIGHTS_READ", STANDARD_RIGHTS_READ);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "FILE_APPEND_DATA", FILE_APPEND_DATA);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "FILE_WRITE_ATTRIBUTES", FILE_WRITE_ATTRIBUTES);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "FILE_WRITE_DATA", FILE_WRITE_DATA);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "FILE_WRITE_EA", FILE_WRITE_EA);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "STANDARD_RIGHTS_WRITE", STANDARD_RIGHTS_WRITE);
  TryOpenFile(L"%SystemRoot%\\Cursors\\arrow_i.cur", output, "WRITE_DAC", WRITE_DAC);

  //%AllUsersProfile%
  TryOpenFile(L"%AllUsersProfile%", output, "ZERO", 0x0);
  TryOpenFile(L"%AllUsersProfile%", output, "GENERIC_READ", GENERIC_READ);
  TryOpenFile(L"%AllUsersProfile%", output, "GENERIC_WRITE", GENERIC_WRITE);
  TryOpenFile(L"%AllUsersProfile%", output, "FILE_ADD_FILE", FILE_ADD_FILE);
  TryOpenFile(L"%AllUsersProfile%", output, "FILE_ADD_SUBDIRECTORY", FILE_ADD_SUBDIRECTORY);
  TryOpenFile(L"%AllUsersProfile%", output, "FILE_APPEND_DATA", FILE_APPEND_DATA);
  TryOpenFile(L"%AllUsersProfile%", output, "FILE_CREATE_PIPE_INSTANCE", FILE_CREATE_PIPE_INSTANCE);
  TryOpenFile(L"%AllUsersProfile%", output, "FILE_DELETE_CHILD", FILE_DELETE_CHILD);
  TryOpenFile(L"%AllUsersProfile%", output, "FILE_LIST_DIRECTORY", FILE_LIST_DIRECTORY);
  TryOpenFile(L"%AllUsersProfile%", output, "FILE_READ_ATTRIBUTES", FILE_READ_ATTRIBUTES);
  TryOpenFile(L"%AllUsersProfile%", output, "FILE_READ_DATA", FILE_READ_DATA);
  TryOpenFile(L"%AllUsersProfile%", output, "FILE_READ_EA", FILE_READ_EA);
  TryOpenFile(L"%AllUsersProfile%", output, "FILE_TRAVERSE", FILE_TRAVERSE);
  TryOpenFile(L"%AllUsersProfile%", output, "FILE_WRITE_ATTRIBUTES", FILE_WRITE_ATTRIBUTES);
  TryOpenFile(L"%AllUsersProfile%", output, "FILE_WRITE_DATA", FILE_WRITE_DATA);
  TryOpenFile(L"%AllUsersProfile%", output, "FILE_WRITE_EA", FILE_WRITE_EA); 
  TryOpenFile(L"%AllUsersProfile%", output, "WRITE_DAC", WRITE_DAC); 

  //%UserProfile%
  TryOpenFile(L"%UserProfile%", output, "ZERO", 0x0);
  TryOpenFile(L"%UserProfile%", output, "GENERIC_READ", GENERIC_READ);
  TryOpenFile(L"%UserProfile%", output, "GENERIC_WRITE", GENERIC_WRITE);
  TryOpenFile(L"%UserProfile%", output, "FILE_ADD_FILE", FILE_ADD_FILE);
  TryOpenFile(L"%UserProfile%", output, "FILE_ADD_SUBDIRECTORY", FILE_ADD_SUBDIRECTORY);
  TryOpenFile(L"%UserProfile%", output, "FILE_APPEND_DATA", FILE_APPEND_DATA);
  TryOpenFile(L"%UserProfile%", output, "FILE_CREATE_PIPE_INSTANCE", FILE_CREATE_PIPE_INSTANCE);
  TryOpenFile(L"%UserProfile%", output, "FILE_DELETE_CHILD", FILE_DELETE_CHILD);
  TryOpenFile(L"%UserProfile%", output, "FILE_LIST_DIRECTORY", FILE_LIST_DIRECTORY);
  TryOpenFile(L"%UserProfile%", output, "FILE_READ_ATTRIBUTES", FILE_READ_ATTRIBUTES);
  TryOpenFile(L"%UserProfile%", output, "FILE_READ_DATA", FILE_READ_DATA);
  TryOpenFile(L"%UserProfile%", output, "FILE_READ_EA", FILE_READ_EA);
  TryOpenFile(L"%UserProfile%", output, "FILE_TRAVERSE", FILE_TRAVERSE);
  TryOpenFile(L"%UserProfile%", output, "FILE_WRITE_ATTRIBUTES", FILE_WRITE_ATTRIBUTES);
  TryOpenFile(L"%UserProfile%", output, "FILE_WRITE_DATA", FILE_WRITE_DATA);
  TryOpenFile(L"%UserProfile%", output, "FILE_WRITE_EA", FILE_WRITE_EA); 
  TryOpenFile(L"%UserProfile%", output, "WRITE_DAC", WRITE_DAC);

  //%Temp%
  TryOpenFile(L"%Temp%", output, "ZERO", 0x0);
  TryOpenFile(L"%Temp%", output, "GENERIC_READ", GENERIC_READ);
  TryOpenFile(L"%Temp%", output, "GENERIC_WRITE", GENERIC_WRITE);
  TryOpenFile(L"%Temp%", output, "FILE_ADD_FILE", FILE_ADD_FILE);
  TryOpenFile(L"%Temp%", output, "FILE_ADD_SUBDIRECTORY", FILE_ADD_SUBDIRECTORY);
  TryOpenFile(L"%Temp%", output, "FILE_APPEND_DATA", FILE_APPEND_DATA);
  TryOpenFile(L"%Temp%", output, "FILE_CREATE_PIPE_INSTANCE", FILE_CREATE_PIPE_INSTANCE);
  TryOpenFile(L"%Temp%", output, "FILE_DELETE_CHILD", FILE_DELETE_CHILD);
  TryOpenFile(L"%Temp%", output, "FILE_LIST_DIRECTORY", FILE_LIST_DIRECTORY);
  TryOpenFile(L"%Temp%", output, "FILE_READ_ATTRIBUTES", FILE_READ_ATTRIBUTES);
  TryOpenFile(L"%Temp%", output, "FILE_READ_DATA", FILE_READ_DATA);
  TryOpenFile(L"%Temp%", output, "FILE_READ_EA", FILE_READ_EA);
  TryOpenFile(L"%Temp%", output, "FILE_TRAVERSE", FILE_TRAVERSE);
  TryOpenFile(L"%Temp%", output, "FILE_WRITE_ATTRIBUTES", FILE_WRITE_ATTRIBUTES);
  TryOpenFile(L"%Temp%", output, "FILE_WRITE_DATA", FILE_WRITE_DATA);
  TryOpenFile(L"%Temp%", output, "FILE_WRITE_EA", FILE_WRITE_EA); 
  TryOpenFile(L"%Temp%", output, "WRITE_DAC", WRITE_DAC); 

  //%AppData%
  TryOpenFile(L"%AppData%", output, "ZERO", 0x0);
  TryOpenFile(L"%AppData%", output, "GENERIC_READ", GENERIC_READ);
  TryOpenFile(L"%AppData%", output, "GENERIC_WRITE", GENERIC_WRITE);
  TryOpenFile(L"%AppData%", output, "FILE_ADD_FILE", FILE_ADD_FILE);
  TryOpenFile(L"%AppData%", output, "FILE_ADD_SUBDIRECTORY", FILE_ADD_SUBDIRECTORY);
  TryOpenFile(L"%AppData%", output, "FILE_APPEND_DATA", FILE_APPEND_DATA);
  TryOpenFile(L"%AppData%", output, "FILE_CREATE_PIPE_INSTANCE", FILE_CREATE_PIPE_INSTANCE);
  TryOpenFile(L"%AppData%", output, "FILE_DELETE_CHILD", FILE_DELETE_CHILD);
  TryOpenFile(L"%AppData%", output, "FILE_LIST_DIRECTORY", FILE_LIST_DIRECTORY);
  TryOpenFile(L"%AppData%", output, "FILE_READ_ATTRIBUTES", FILE_READ_ATTRIBUTES);
  TryOpenFile(L"%AppData%", output, "FILE_READ_DATA", FILE_READ_DATA);
  TryOpenFile(L"%AppData%", output, "FILE_READ_EA", FILE_READ_EA);
  TryOpenFile(L"%AppData%", output, "FILE_TRAVERSE", FILE_TRAVERSE);
  TryOpenFile(L"%AppData%", output, "FILE_WRITE_ATTRIBUTES", FILE_WRITE_ATTRIBUTES);
  TryOpenFile(L"%AppData%", output, "FILE_WRITE_DATA", FILE_WRITE_DATA);
  TryOpenFile(L"%AppData%", output, "FILE_WRITE_EA", FILE_WRITE_EA); 
  TryOpenFile(L"%AppData%", output, "WRITE_DAC", WRITE_DAC);

  //CV
  //TryWriteFile(L"%SystemDrive%", output);
  //TryWriteFile(L"%SystemRoot%", output);
  //TryWriteFile(L"%ProgramFiles%", output);
  //TryWriteFile(L"%SystemRoot%\\System32", output);
  //TryWriteFile(L"%AllUsersProfile%", output);
  //TryWriteFile(L"%UserProfile%", output);
  //TryWriteFile(L"%Temp%", output);
  //TryWriteFile(L"%AppData%", output);
}
