#!/usr/bin/env ruby
#
# Google Chrome Stable updates blog scraper
#
# Author: Joshua J. Drake / Accuvant LABS
# License: BSD
#

require 'date'

require 'nokogiri'


require 'mysql'
$db = Mysql::new("localhost", "root", "", "browsers")

#$debug = true


def insert_vuln(milestone_id, title, vendor_id, cve, impact, reported, discovered, disclosed)
	return if milestone_id.nil?

	values = "'3'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(title) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(vendor_id) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(cve) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(impact) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(reported) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(disclosed) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(discovered) + "'"
	values << ", " if values.length > 0
	values << "'" + milestone_id.to_s + "'"
	
	qstr = "insert into vulns (prod_id, title, vendor_id, cve, severity, report_date, disclosure_date, discover_date, advisory_id)"
	qstr << " values (" << values << ")"
	$db.query(qstr)
end


def insert_milestone(title, vendor_id, publish)
	values = "'3'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(title) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(vendor_id) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(publish) + "'"
	
	qstr = "insert into advisories (prod_id, title, vendor_id, publish_date)"
	qstr << " values (" << values << ")"
	$db.query(qstr)
	return $db.insert_id()
end


#def show_vuln(vuln)
#end


def show_post(post)
	puts "=" * 80
	puts "Milestone: " + post[:ms].inspect
	puts "Title: " + post[:title].inspect
	puts "Publish date: " + post[:date].inspect
	puts "URL: " + post[:href].inspect
	puts ""

	# Build the friendly title/milestone name
	title = post[:title] + " - v" + post[:vers]
	ms_str = post[:ms]

	# Insert the "advisory"
	ms_id = insert_milestone(title, ms_str, post[:date].to_s)

	# Print/parse the body of the post
	line = ""
	body = ""
	vers = nil
	buglist = []
	post[:body].each { |ln|
		if ln.kind_of?(String)
			line << " " if line.length > 0
			line << ln
			#$stderr.puts ln.inspect if $debug

		elsif ln.kind_of?(Hash)
			line << " " if line.length > 0
			line << ln[:text]
			#$stderr.puts ln.inspect if $debug

		elsif not ln
			body << line
			body << "\n"

			# Vuln!!
			parts = line.split('[')
			if parts.length > 1
				if $debug
					$stderr.puts "LINE: " + line.inspect 
					$stderr.puts ""
				end
				vuln = {}

				# Slice off the empty first element
				parts.slice!(0,1)

				parts.each { |part|
					$stderr.puts "  PART: " + part.inspect if $debug
					# Skip anything that isn't a bug number.. This includes reward
					# amounts, since we don't need these.
					part.lstrip!
					next if (part[0,1] !~ /[0-9]/)
					next if part[0,6] == "64-bit"

					# Some weird references
					next if part.index(' ].')

					# Now we have a bug number beginning in "first"
					hrm = part.split(' ] ')

					# If there's only one, we probably found a bug list..
					if hrm.length == 1
						buglist << hrm.first
						$stderr.puts "  BUGLIST: " + buglist.inspect if $debug
						next
					end

					# If we're still here we check for invalid stuff..
					raise "Invalid number of closing brackets (#{hrm.length})!" if hrm.length != 2

					# Okay, save the bug and move on to the "rest"
					vuln[:bug],rest = hrm

					# Skip a special case of a bug mentioned but not fixed.
					if post[:vers] == '4.1.249.1036' and
							vuln[:bug] == '33572'
						next
					end

					# The first word should be the severity, pop it off
					hrm = rest.split(' ')
					severity = hrm.shift
					case severity
					when "Critical", "High", "Medium", "Low"
						# These are ok

					else
						$stderr.puts "Invalid severity encountered: #{severity.inspect}"
						next

					end

					# The CVE, if it exists, will be next
					cve = ''
					if hrm.first =~ /^(CVE-....-....)/
						cve = $1
						hrm.shift
					end
					#$stderr.puts " CVE: " + cve.inspect

					# See if we can find a credit statement
					if (cidx = hrm.index("Credit:")) or
						(cidx = hrm.index("Credited")) or
						((cidx = hrm.index("Credit")) and ((hrm[cidx+1] == "to") or (hrm[cidx+1] == ":")))
						cred = hrm.slice!(cidx, hrm.length)

						# Strip the "Credit.*" heading
						cred.shift
						if cred[0] == "Credit"
							cred.shift
						end

						vuln[:credit] = cred.join(' ')
					end

					# The remaining stuff is the title
					vuln[:title] = hrm.join(' ')

					# If we have a buglist, we make some dupes and append them
					if buglist.length > 0
						buglist << vuln[:bug]
						$stderr.puts "  FINAL BUGLIST: " + buglist.inspect if $debug
						buglist.each_with_index { |bug,vidx|
							v = vuln.clone
							bug_title = vuln[:title] + " \##{vidx+1}"

							#show_vuln(vuln)

							# Add it to the database
							insert_vuln(ms_id,
								bug_title,
								bug,
								cve,
								severity,
								'', '', post[:date].to_s)
						}
						buglist = []
					else
						#show_vuln(vuln)

						# Add it to the database
						insert_vuln(ms_id,
							vuln[:title],
							vuln[:bug],
							cve,
							severity,
							'', '', post[:date].to_s)
					end
				}
			end

			line = ""
			#puts ""

		else
			puts "UNKNOWN TYPE: " + ln.class.to_s

		end
	}
	body << line if line.length > 0
	puts body

	# Insert manual bugs that didn't get recorded
	case post[:vers]
	when 'WILL-NEVER-HAPPEN'
		insert_vuln(ms_id, '',
			'', '', '',
			'', '', post[:date].to_s)

	when '1.0.154.46'
		# No bug number for these two (mentions Internal issue 1543405)
		insert_vuln(ms_id, 'Adobe Reader Plugin Open Parameters DoS',
			'', 'CVE-2007-0048', 'Medium',
			'', '', post[:date].to_s)
		insert_vuln(ms_id, 'Adobe Reader Plugin Open Parameters Cross-Site Scripting Vulnerability',
			'', 'CVE-2007-0045', 'Medium',
			'', '', post[:date].to_s)
		# end-of-no-bug-section
		insert_vuln(ms_id, 'Javascript Same-Origin Bypass',
			'6264', 'CVE-2009-0276', 'High',
			'2009-01-11', '', post[:date].to_s)

	when '1.0.154.48'
		# Missing severity rating / Bug closed
		insert_vuln(ms_id, 'command line arguments could be injected and executed',
			# Similar to CVE-2007-3670
			'5825', '', '',
			'', '', post[:date].to_s)

	when '1.0.154.59'
		insert_vuln(ms_id, 'ChromeHTML protocol handler same-origin bypass',
			'9860', 'CVE-2009-1412', 'High',
			'2009-04-08', '', post[:date].to_s)

	when '1.0.154.64'
		insert_vuln(ms_id, 'Input validation error in the browser process.',
			'10869', 'CVE-2009-1441', 'Critical',
			'2009-04-22', '', post[:date].to_s)
		insert_vuln(ms_id, 'Integer overflow in Skia 2D graphics.',
			'10736', 'CVE-2009-1442', 'High',
			'2009-04-19', '', post[:date].to_s)

	when '1.0.154.65'
		insert_vuln(ms_id, 'Denial of service in SVG',
			'9019', 'CVE-2009-0945', 'High',
			'2009-03-19', '', post[:date].to_s)

	when '2.0.172.31'
		insert_vuln(ms_id, 'Memory corruption',
			'', 'CVE-2009-1690', 'High',
			'', '', post[:date].to_s)
		insert_vuln(ms_id, 'Drag and drop information leak',
			'', 'CVE-2009-1718', 'Medium',
			'', '', post[:date].to_s)

	when '2.0.172.33'
		insert_vuln(ms_id, 'Buffer overflow processing HTTP responses',
			'14508', 'CVE-2009-2121', 'Critical',
			'2009-06-17', '', post[:date].to_s)

	when '2.0.172.37'
		insert_vuln(ms_id, 'Heap overflow with Javascript regular expressions',
			'14719', 'CVE-2009-2555', 'High',
			'2009-06-19', '', post[:date].to_s)
		# Couldn't find bug number
		insert_vuln(ms_id, 'Memory corruption in the browser process',
			'', 'CVE-2009-2556', 'Critical',
			'', '', post[:date].to_s)

	when '2.0.172.43'
		insert_vuln(ms_id, 'Unauthorized memory read from Javascript',
			'18639', 'CVE-2009-2935', 'High',
			'2009-08-06', '', post[:date].to_s)
		insert_vuln(ms_id, 'Treat weak signatures as invalid',
			'18725', '', 'Medium',
			'2009-08-06', '', post[:date].to_s)
		insert_vuln(ms_id, 'Stack consumption vulnerability in libxml2',
			'19158', 'CVE-2009-2414', 'Low',
			'2009-08-12', '', post[:date].to_s)
		insert_vuln(ms_id, 'Multiple use-after-free vulnerabilities in libxml2',
			'19158', 'CVE-2009-2416', 'High',
			'2009-08-12', '', post[:date].to_s)

	when '3.0.195.21'
		insert_vuln(ms_id, 'Content-Type: application/rss+xml being rendered as active content',
			'21238', '', 'Medium',
			'2009-09-07', '', post[:date].to_s)
		insert_vuln(ms_id, 'Same Origin Policy Bypass via getSVGDocument() method',
			'21338', '', 'High',
			'2009-09-08', '', post[:date].to_s)

	when '3.0.195.24'
		# Couldn't find a bug #
		insert_vuln(ms_id, 'dtoa() error parsing long floating point numbers',
			'', 'CVE-2009-0689', 'High',
			'', '', post[:date].to_s)

	when '3.0.195.27'
		insert_vuln(ms_id, 'avcodec-52.dll is not marked NX, SafeSEH or DBCompat',
			'23189', '', 'Low',
			'2009-09-26', '', post[:date].to_s)

	when '3.0.195.32'
		insert_vuln(ms_id, 'User not warned for some file types that can execute JavaScript',
			'23979', '', 'Medium',
			'2009-10-06', '', post[:date].to_s)
		insert_vuln(ms_id, 'Possible memory corruption in the Gears plugin',
			'26179', '', 'High',
			'2009-10-28', '', post[:date].to_s)

		
	when '3.0.195.33'
		# Couldn't find a bug #
		insert_vuln(ms_id, 'Custom headers incorrectly sent for CORS OPTIONS request',
			'', 'CVE-2009-2816', 'Low',
			'', '', post[:date].to_s)

	end

	puts ""
end


def scrape_children(node, post)
	ret = []

	txt = node.text.strip
	return ret if txt.length < 1

	node.children.each { |c|
		case c.name

		when "br"
			ret << nil if ret.last

		when "a"
			link = {}
			link[:href] = c['href']
			link[:text] = c.text.strip
			ret << link

		when "text"
			txt = c.text.strip
			if txt.length < 1
				ret << nil if ret.last
			else
				txt = txt.split("\u00a0")
				txt.each { |el|
					el.strip!
					if not post[:vers] and el =~ /([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)[,. ]/
						post[:vers] = $1
					end
				}
				ret += txt
				#ret << txt
			end

		when "span", "div", "p", "ul", "li", "b", "strong", "i", "em", "blockquote", "u", "ol", "h3", "use"
			ret += scrape_children(c, post)

		else
			$stderr.puts "Unknown tag #{c.name.inspect} inside <#{node.name}>:" + c.inspect

		end
	}

	ret
end


#
# Scrape a single post
#
def scrape_post(post, node)
	post[:body] = []

	node.children.each { |c|
		case c.name

		when "text"
			next if c.text == "\n"
			raise 'text found!'

		when 'a'
			# ignore, they are empty anyway

		when 'p'
			next if c['class'] != 'post-subhead'
			next if c.text =~ /Labels:/

			date_text = c.text.strip.gsub(/ \| /, ' ')
			$stderr.puts "\n===========================" if $debug
			$stderr.puts "DATE: " + date_text.inspect if $debug
			post[:date] = Date.parse(date_text)

		when 'h3'
			# Scrape the post title/url
			c.children.each { |h3c|
				next if h3c.name != 'a'
				post[:href] = h3c['href']
				post[:title] = h3c.text
			}

		when 'div'
			next if c['class'] != 'post-body'

			txt = scrape_children(c, post)
			$stderr.puts "RET:" + txt.join("\n") if $debug
			post[:body] += txt

		else
			$stderr.puts "Unexpected tag #{c.name.inspect} while processing post"

		end
	}
	#puts post.inspect
	post
end


#
# Scrape all posts from the Google Chrome blog local copy
#
def get_posts()
	ret = []

   fns = []
	Dir.glob('posts/*.html') { |f|
		fns << f
	}

	fns.each { |f|
		html = nil

		$stderr.puts "*** Processing #{f} ..." if $debug
		break if not File.exists?(f)
		File.open(f, "rb") { |fd|
			html = fd.read
		}

		doc = Nokogiri::HTML(html)
		#puts doc.inspect

		# Process posts
		new_post = {}

		# Derive milestone from the filename
		new_post[:ms] = f.split(/posts\/|.html/)[1]

		# Get the rest of the info
		doc.xpath('//div[@class = "post"]').each { |node|
			scrape_post(new_post, node)
			#show_post(new_post)
			ret << new_post
		}
	}
	ret
end



posts = get_posts()

posts.sort! { |a,b| a[:ms] <=> b[:ms] }

posts.each { |post|
	#next if post[:title] !~ /Stable.*Update/
	show_post(post)
}

