#!/usr/bin/env ruby
#
# Mozilla Firefox Bugzilla scraper
#
# This script simple snarfs out the reported date and the title for
# the provided bug numbers.
#
# Author: Joshua J. Drake / Accuvant LABS
# License: BSD
#
#
# Mozilla data is available under a Creative Commons License. For
# more information, see:
#
# http://www.mozilla.org/foundation/licensing/website-content.html
#
#


require 'date'

require 'nokogiri'

require 'mysql'
$db = Mysql::new("localhost", "root", "", "browsers")

#$debug = true


#
# Scrape all the bug data from the local data store
#
def get_bugs()
	ret = []

	fns = []
	Dir.glob('**/*.html') { |f|
		fns << f
	}

	fns.each { |f|
		html = nil

		$stderr.puts "*** Processing: #{f}" if $debug
		File.open(f, "rb") { |fd|
			html = fd.read
		}

		doc = Nokogiri::HTML(html)
		#puts doc.inspect

		# Process bugs
		new_bug = {}
		doc.xpath('//title').each { |node|
			raise 'title already found' if new_bug[:title]

			if node.text =~ /Bug (\d+) . (.*)/
				bug = $1
				title = $2

				new_bug[:title] = title
				new_bug[:bug] = bug

			elsif node.text == "Access Denied"
				$stderr.puts "Bug #{f} was inaccessible..."

			else
				raise 'wtf?'

			end
		}

		# No bug # found?
		next unless new_bug[:bug]

		doc.xpath('//td').each { |node|
			next unless node.children.length == 3
			txt = node.text.strip
			next unless txt =~ /(.* P[SD]T) by /

			raise 'reported date already found!' if new_bug[:reported]
			$stderr.puts "Reported: " + $1.inspect if $debug

			# This is the thing we wanted most..
			new_bug[:reported] = Date.parse($1)
		}
		ret << new_bug
		#break
	}
	ret
end

bugs = get_bugs()

bugs.each { |bug|
	puts bug.inspect

	# update the database with this new information
	bug_id = $db.escape_string(bug[:bug])
	rep_date = $db.escape_string(bug[:reported].to_s)

	qstr = "update vulns set report_date='#{rep_date}' where prod_id='1' and vendor_id='#{bug_id}'"
	$db.query(qstr)
}
