#!/usr/bin/env ruby
#
# Microsoft Security Bulletin Search results scraper
#
# Author: Joshua J. Drake / Accuvant LABS
# License: BSD
#
#
# Some notes are inline...
#


require 'date'

require 'nokogiri'

require 'mysql'
$db = Mysql::new("localhost", "root", "", "browsers")

#$debug = true


def insert_vuln(msb_id, title, vendor_id, cve, impact, reported, discovered, disclosed)
	values = "'2'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(title) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(vendor_id) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(cve) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(impact) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(reported) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(disclosed) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(discovered) + "'"
	values << ", " if values.length > 0
	values << "'" + msb_id.to_s + "'"
	
	qstr = "insert into vulns (prod_id, title, vendor_id, cve, severity, report_date, disclosure_date, discover_date, advisory_id)"
	qstr << " values (" << values << ")"
	$db.query(qstr)
end


def insert_msb(title, vendor_id, publish)
	values = "'2'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(title) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(vendor_id) + "'"
	values << ", " if values.length > 0
	values << "'" + $db.escape_string(publish) + "'"
	
	qstr = "insert into advisories (prod_id, title, vendor_id, publish_date)"
	qstr << " values (" << values << ")"
	$db.query(qstr)
	return $db.insert_id()
end



def show_msb(msb)
	puts "=" * 80
	puts "MSB: " + msb[:msb].inspect
	puts "Title: " + msb[:title].inspect
	puts "Subtitle: " + msb[:subtitle].inspect
	puts "KB Article: " + msb[:kb].inspect
	puts "Date: " + msb[:date].to_s
	puts "Severity: " + msb[:severity].inspect
=begin
	puts "Products: "
	msb[:prod].each { |p|
		p.each_pair { |k,v|
			if v.kind_of? Hash
				puts "  #{k}:"
				puts "    " + v[:text].inspect
				puts "    " + v[:href].inspect
			else
				puts "  #{k}: " + v.inspect
			end
		}
	}
=end

=begin
	puts "Products (w/o SP): "
	msb[:prod2].sort.each { |p|
		puts "  " + p
	}

	if msb[:body]
		puts ""
		puts "Description: " + msb[:body].join("\n\n")
		puts ""
	end
=end

	msb_id = insert_msb(msb[:subtitle], msb[:msb], msb[:date].to_s)

	# Last minute fix ups for a special cases.
	case msb[:msb]
	when 'MS11-031'
		msb[:vulns]["Scripting Memory Reallocation Vulnerability - CVE-2011-0663"].each { |v|
			v << "Critical"
			break
		}
	end

	puts "Vulns:"
	if $debug
		$stderr.puts "COLZ:"
		$stderr.puts msb[:vcols].join("\n")
		$stderr.puts ""
	end

	msb[:vcols].each { |v|
		case v

		when "Affected Software", "Aggregate Severity Rating", "Software", "Operating System", "Component"
			# ignore

		else
			puts "  Title: " + v.inspect

			# Extract the CVE identifier
			cve = ''
			if v =~ /- (CVE-[0-9]{4}-[0-9]{4})$/
				cve = $1
			else
				raise "Unknown non-vulnerability in columns from SR & VI section"
			end
			puts "  CVE: " + cve.inspect

			# Extract the title
			bug_title,rest = v.split('- ')

			# Determine max severity..
			severity = 'Not affected'

			# Lookup the data for this column
			col = msb[:vulns][v]
			col.each { |td|
				td.each { |el|
					el.gsub!("\u00A0", "")

					case el

					when "Critical"
						severity = el

					when "Important"
						severity = el if severity != 'Critical'

					when "Moderate"
						severity = el if not [ 'Critical', 'Important' ].include? severity

					when "Low"
						severity = el if severity == 'Not affected'

					when ""
						# ignore
					when "Remote Code Execution", "Information Disclosure", "Elevation of Privilege"
						# ignore
					when "Denial of Service", "Tampering", "Spoofing"
						# ignore
					when "None", "N/A", "Not applicable", "No severity rating", "None**", "Not affected"
						# ignore

					else
						$stderr.puts "Unknown field #{el.inspect} encountered when determining severity"

					end
				}
			}
			puts "  Severity: " + severity.inspect

			# Add it to the database
			insert_vuln(msb_id,
				bug_title,
				'', # Microsoft doesn't give these out
				cve,
				severity,
				'', '', msb[:date].to_s)

			puts ""
		end
	}

	puts ""
end


INCLUDE = [
	".NET Framework and Microsoft Silverlight",
	"Canonical Display Driver",
	"DHTML Editing Component ActiveX Control",
	"GDI",  # MS08-071
	"Help and Support Center",
	"JScript Scripting Engine",
	"Microsoft .NET Common Language Runtime and in Microsoft Silverlight",
	"Microsoft .NET Common Language Runtime",
	"Microsoft Active Template Library (ATL)",  # MS09-037 - Windows/Browers version
	"Microsoft Data Access Components",
	"Microsoft Windows Image Color Management System",
	"OLE Automation",  # MS11-038
	"Remote Desktop Connection",  # MS09-044 - ActiveX client
	"SChannel could allow Remote Code Execution",
	"Security Update for Internet Explorer",
	"Unicode Scripts Processor",
	"VBScript Scripting Engine",
	"Vector Markup Language",
	"Windows Common Control Library",
	"Windows CryptoAPI",
	"Windows Shell",  # MS10-046 - stuxnet/icon loader
	"Windows Shell Handler",  # MS10-007
	"JScript and VBScript Scripting Engines",
	"MHTML",
	"Microsoft XML Core Services",
	"SChannel",
	".NET Framework",  # MS10-077, MS11-028, MS11-044 - metion IE
	"Embedded OpenType Font Engine",
	"GDI+",
	"Cumulative Security Update of ActiveX Kill Bits",
	"Cumulative Security Update for Internet Explorer",
]

EXCLUDE = [
	"Active Directory Certificate Services Web Enrollment",  # Server-side reflected XSS
	"ActiveX Control for the Snapshot Viewer for Microsoft Access",  # Comes with Office
	"Ancillary Function Driver",
	"ASP.NET in Microsoft Windows",
	"ASP.NET",
	"Blended Threat Vulnerability in SearchPath",  # CVE already included in IE bulletin
	"COM Validation in Microsoft Office",
	"COM Validation in Windows Shell and WordPad",
	"Consent User Interface",
	"Distributed File System",
	"DNS",
	"DNS Resolution",
	"DNS and WINS Server",
	"Event System",
	"Forefront Unified Access Gateway (UAG)",
	"FTP Service for Internet Information Services",
	"Host Integration Server RPC Service",
	"IPsec Policy Processing",
	"Indexing Service",
	"Internet Authentication Service",
	"Insecure Library Loading in Internet Connection Signup Wizard",
	"Internet Information Services (IIS) FTP Service",
	"Internet Information Services (IIS)",
	"Internet Information Services",
	"License Logging Server",
	"Microsoft Active Template Library (ATL) ActiveX Controls for Microsoft Office",
	"Microsoft Ancillary Function Driver",
	"Microsoft Exchange Server",
	"Microsoft Exchange and Windows SMTP Service",
	"Microsoft Exchange",
	"Microsoft Foundation Class (MFC) Library",
	"Microsoft Foundation Classes",
	"Microsoft Groove",
	"Microsoft ISA Server 2006",
	"Microsoft ISA Server and Forefront Threat Management Gateway (Medium Business Edition)",
	"Microsoft Internet Information Services (IIS)",
	"Microsoft Office (MSO)",
	"Microsoft Office Access ActiveX Controls",
	"Microsoft Office Filters",
	"Microsoft Office Graphics Filters",
	"Microsoft Office Outlook",
	"Microsoft Office Project",
	"Microsoft Office SharePoint Server",
	"Microsoft Office Visio",
	"Microsoft Office Web Components",
	"Microsoft Outlook",
	"Microsoft Paint",
	"Microsoft Publisher",
	"Microsoft Visual Basic for Applications",
	"Microsoft Windows",  # MS10-095
	"Microsoft Windows Media Services",
	"Microsoft Works Converters",
	"Microsoft XML Editor",
	"Media Player Network Sharing Service",
	"OpenType Font (OTF) Driver",  # MS10-091 - IE not affected, Explorer preview req'd
	"OpenType Font (OTF) Format Driver",  # MS10-078 - IE not affected?
	"Outlook Express and Windows Mail",
	"Outlook Web Access for Exchange Server",
	"Pragmatic General Multicast (PGM)",
	"Print Spooler Service",
	"Remote Desktop Client",  # MS11-017 - dll loading
	"Remote Procedure Call",
	"Routing and Remote Access",
	"RPC",
	"SafeHTML",  # affects Sharepoint
	"Security Update for Outlook Express and Windows Mail",
	"Server Service",
	"SMBv2",
	"Task Scheduler",
	"TCP/IP",
	"Telnet",
	"Threat Management Gateway Firewall Client",
	"Tracing Feature for Services",
	"Virtual Address Descriptor Manipulation",
	"Virtual PC and Virtual Server",
	"Visual Basic 6.0 Runtime Extended Files (ActiveX Controls)",  # comes with office/visual studio
	"Visual Studio Active Template Library",  # for redist runtime
	"Web Services on Devices API",
	"Windows Address Book",
	"Windows Backup Manager",
	"Windows Client/Server Runtime Subsystem",
	"Windows Explorer",
	"Windows Fax Cover Page Editor",
	"Windows HTTP Services",
	"Windows ISATAP Component",
	"Windows Internet Printing Service",
	"Windows Local Procedure Call",
	"Windows Messenger",
	"Windows Media",  # MS11-015 - one dll loading, one not *in* IE
	"Windows Media Encoder 9",  # non-default
	"Windows Media Encoder",   # non-default
	"Windows Netlogon Service",
	"Windows Print Spooler",
	"Windows Server 2008 Hyper-V",
	"Windows Shared Cluster Disks",
	"Windows Shell Graphics Processing",  # Explorer preview
	"Wireless LAN AutoConfig Service",
	"Workstation Service",
	"Bluetooth Stack",
	"Hyper-V",
	"Kerberos",
	"Message Queuing",
	"Microsoft MPEG Layer-3 Codecs",
	"Microsoft Office PowerPoint",
	"Microsoft Office Publisher",
	"Microsoft SQL Server",
	"Microsoft SharePoint",
	"Microsoft Word",
	"OpenType Compact Font Format (CFF) Driver",  # IE not mentioned, but third party browsers are in MS11-007, MS11-032 (not MS10-037, too old?)
	"Windows",  # MS10-019 CAB files / MS09-012 Service Isolation
	"Windows Search",
	"Windows TCP/IP",
	"WordPad Text Converters",
	"WordPad and Office Text Converters",
	"Microsoft DirectShow",
	"Microsoft Visio",
	"SMB",
	"WINS",
	"Windows Client/Server Run-time Subsystem",
	"Windows Movie Maker",
	"Local Security Authority Subsystem Service",
	"Microsoft Office Word",
	"Microsoft Office",
	"Microsoft PowerPoint",
	"SMB Client",
	"SMB Server",
	"Windows Media Player",
	"Active Directory",
	"Microsoft Excel",
	"Microsoft Office Excel",
	"Windows Kernel",
	"Windows Kernel-Mode Drivers",
]

DUNNO = [
	"Active Directory Federation Services",  # using cache to bypass SSO
	"Cinepak Codec",  # mentions IE
	"DirectX",  # mentions IE
	"MPEG-4 Codec",  # mentions IE
	"Media Decompression",  # mentions streaming content via web site
	"Microsoft .NET Framework",  # MS10-041 - xml HMAC bug
	"Windows Media Components",  # MS08-076 - SPN/ISATAP relay
	"Windows Media File Processing",  # MS09-038 - mentions IE
	"Windows Media Format",  # MS09-047 - mentions IE
	"Windows Media Runtime",  # MS09-051 - mentions IE
]


def include_msb(st)
	# hax!
	#return true

	st = st.dup
	st.gsub!(/ \([0-9]*\)$/, '')
	st.gsub!(/^Vulnerabilit[^ ]* in /, '')
	st.gsub!(/^the /, '')
	st.gsub!(/ Could .*/, '')

	#$stderr.puts st.inspect if $debug

	INCLUDE.include?(st) or DUNNO.include?(st)
end


def split_title(title)
	first = severity = nil
	if title.include? "\u2013"
		first,severity = title.split(" \u2013 ")
	else
		first,severity = title.split(' - ')
	end
	msb = first.gsub!('Microsoft Security Bulletin ', '')
	[ msb, severity ]
end


def scrape_table(node, msb)
	# skip tables we dont care about
	return if not [ 
		"Severity Ratings and Vulnerability Identifiers",
		"Affected and Non-Affected Software"
		].include? msb[:subsection]

	cols = []
	rows = []
	node.children.each { |c2|
		case c2.name

		when "thead"
			c2.children.each { |c3|
				if c3.name != "tr"
					$stderr.puts "Unexpected tag \"#{c3.name}\" encountered inside thead"
					next
				end
				c3.children.each { |c4|
					if c4.name != "td"
						$stderr.puts "Unexpected tag \"#{c4.name}\" encountered inside thead/tr"
						next
					end

					# Skip this table heading as its not really a column label
					next if c4['class'] == "tableHeader"

					c4.children.each { |c5|
						if c5.name != "text"
							$stderr.puts "Unexpected tag \"#{c5.name}\" encountered inside thead/tr/td"
							next
						end
						txt = c5.text.strip
						txt.gsub!("\u2013", "-")
						$stderr.puts "        " + txt.inspect if $debug
						cols << txt
					}
				}
			}

		when "tbody"
			c2.children.each { |c3|
				if c3.name != "tr"
					$stderr.puts "Unexpected tag \"#{c3.name}\" encountered inside tbody"
					next
				end
				
				# Skip these, they don't have any useful data
				if c3['class'] == "subHeader"
					next
				end
				
				row = {}
				cidx = 0

				c3.children.each { |c4|
					if c4.name != "td"
						$stderr.puts "Unexpected tag \"#{c4.name}\" encountered inside tbody/tr"
						next
					end
					c4.children.each { |c5|
						if c5.name != "p"
							$stderr.puts "Unexpected tag \"#{c5.name}\" encountered inside tbody/tr/td"
							next
						end
						c5.children.each { |c6|
							case c6.name

							when "text", "b", "i"
								txt = c6.text.strip
								key = cols[cidx]
								$stderr.puts "        #{key}: #{txt}" if $debug
								row[key] ||= []
								row[key] << txt

							when "a"
								el = {}
								el[:href] = c6['href']
								el[:text] = c6.text.strip
								key = cols[cidx]
								$stderr.puts "        #{key}: #{el.inspect}" if $debug
								row[key] ||= []
								row[key] << el

							when "br"
								# ignore

							when "sup"
								# ignore footnotes

							else
								$stderr.puts "Unexpected tag \"#{c6.name}\" encountered inside tbody/tr/td/p"
								
							end
						}
					}
					cidx += 1
				}
				
				rows << row
			}
			
		when "tr"
			c2.children.each { |c3|
				if c3.name != "td"
					$stderr.puts "Unexpected tag \"#{c3.name}\" encountered inside table/tr"
					next
				end
				c3.children.each { |c4|
					case c4.name
						
					when "p"
						txt = c4.text.strip
						$stderr.puts "        PTXT: " + txt.inspect if $debug
						
					when "text"
						# Ignore? All short?
						txt = c4.text.strip
						next if txt.length < 2
						raise 'nope, not all of them are short...'

					when "script", "h5"
						# definitely ignore..

					when "h4"
						# These have some yummy bits of date, but we'll ignore them for now.

					when "table"
						# These are workaround infos, let's skip for now.
						#scrape_table(c4, msb)
						
					else
						$stderr.puts "Unexpected tag \"#{c4.name}\" encountered inside table/tr/td"
						$stderr.puts "-> " + c4.inspect

					end
				}
			}
			
		else
			$stderr.puts "Unexpected node type \"#{c2.name}\" encountered inside table"
			
		end
	}

	# Record the result in the correct place (based on subsection)
	case msb[:subsection]

	when "Affected and Non-Affected Software"
		$stderr.puts "      PROD-TABLE: " if $debug
		msb[:prod] = rows
		
	when "Acknowledgments"
		msb[:acks] = rows
		
	when "Support"
		msb[:support] = rows

	when "Revisions"
		msb[:revs] = rows

	when "Detection and Deployment Tools and Guidance"
		msb[:detect] = rows

	when "Security Update Deployment"
		msb[:deploy] = rows

	when "Severity Ratings and Vulnerability Identifiers"
		# Merge rows into vulns/vcols
		msb[:vcols] ||= []
		msb[:vulns] ||= {}

		if $debug
			$stderr.puts ""
			$stderr.puts "cols:"
			$stderr.puts cols.join("\n")

			$stderr.puts ""
			$stderr.puts "rows:"
			$stderr.puts rows.join("\n")
		end

		cols.each { |col|
			if not msb[:vcols].index(col)
				msb[:vcols] << col
			end

			if $debug
				$stderr.puts ""
				$stderr.puts "col: " + col.inspect
			end

			# Hash of arrays, by column
			msb[:vulns][col] ||= []

			rows.each { |row|
				#$stderr.puts "   row1: " + row.inspect if $debug
				cur = row[col]
				$stderr.puts "   row2: " + cur.inspect if $debug
				msb[:vulns][col] << cur
			}
		}

	else
#Unknown subsection setting when encountering a table: "HTML Objects Memory Corruption Vulnerability  CVE-2008-1442"
#Unknown subsection setting when encountering a table: "Request Header Cross-Domain Information Disclosure Vulnerability  CVE-2008-1544"
		$stderr.puts "Unknown subsection inside table: \"#{msb[:subsection]}\""
		
	end
end


def scrape_div(node, msb)
	node.children.each { |c|
		case c.name

		when "a", "script"
			# ignore

		when "p"
			$stderr.puts "      TXT1: " + c.text.strip.inspect if $debug

		when "table"
			scrape_table(c, msb)

		when "div"
			case c['class']

			when "expandoIndent"
				c.children.each { |c2|
					case c2.name

					when "table"
						scrape_table(c2, msb)

					when "p"
						if $debug
							#$stderr.puts "  P: " + c2.inspect
							txt = c2.text.strip
							$stderr.puts "      TXT2: " + txt.inspect
						end

					when "a"
						# ignore these, they are just for links to "Top of Section"
						next if not c2['href']
						$stderr.puts "  A: " + c2.inspect if $debug

					when "pre"
						$stderr.puts "      PRE: " + c2.inspect if $debug

					when "div"
						# ignore, it should be picked up by the main xpath query
						#$stderr.puts " DIV2: %-30s %s" % [ c2['id'], c2['class'] ]

					else
						$stderr.puts "Unknown node type \"#{c2.name}\" encountered inside expandoIndent"

					end
				}

			when "dataTableBottomMargin", "secTop"
				# ignore

			else
				$stderr.puts "Unknown div class \"#{c['class']}\" encountered inside div"

			end

		else
			$stderr.puts "Unexpected node type \"#{c.name}\" encountered inside div"

		end
	}
end


#
# Scrape all bulletins from the Microsoft Security Bulletin Search results
#
def get_bulletins()
	ret = []

	fns = []
	Dir.glob('**/*.html') { |f|
		fns << f
	}

	fns.each { |f|
		html = nil
		$stderr.puts "*** Processing: #{f}" if $debug
		begin
			File.open(f, "rb") { |fd|
				html = fd.read
			}
		rescue
		end
		next if not html

		doc = Nokogiri::HTML(html)
		#puts doc.inspect

		# Process bulletin data
		msb = {}
		nodes = doc.xpath('//title')
		raise 'no title found!' if nodes.length < 1
		nodes.each { |node|
			# Scrape the title (includes MSB# and bulletin severity)
			raise 'title already found!' if msb[:title]
			raise 'subtitle already found!' if msb[:subtitle]
			txt = node.text.strip
			txt.gsub!("\u2013", "-")
			msb[:title],subtitle = txt.split(': ')
			msb[:subtitle] = subtitle

			# Skip the bulletin if its not in our list to include
			if not include_msb(subtitle)
				msb.clear
				break
			end

			if subtitle =~ / \(([0-9]+)\)/
				msb[:kb] = $1
			end

			raise 'severity already found!' if msb[:severity]
			raise 'msb already found!' if msb[:msb]
			msb[:msb], msb[:severity] = split_title(msb[:title])
			if not msb[:severity]
				$stderr.puts first.inspect if $debug
				raise 'failed to scrape severity!'
			end
			raise 'invalid msb!' if msb[:msb] !~ /MS[01][8901]-[0-9]{3}/
		}

		# If we have no keys, we skipped this bulletin...
		next if msb.keys.length < 1

		# sanity checking
		nodes = doc.xpath('//meta[@name="BulletinSeverityRating"]')
		raise 'no meta BSR found!' if nodes.length < 1
		nodes.each { |node|
			if node['content'] != msb[:severity]
				raise "Severity misatch found!"
			end
		}

		# Get version information
		nodes = doc.xpath('//meta[@name="Version"]')
		raise 'no meta Version found!' if nodes.length < 1
		nodes.each { |node|
			raise 'already found version!' if msb[:version]
			msb[:version] = node['content']
		}
		nodes = doc.xpath('//meta[@name="LastRevision"]')
		# This doesn't exist sometimes :(
		#raise 'no meta LastRevision found!' if nodes.length < 1
		nodes.each { |node|
			raise 'already found last revision!' if msb[:lastrev]
			msb[:lastrev] = node['content']
		}


		# Scrape the rest of what we need (plus some)
		nodes = doc.xpath('//h1 | //h2 | //h3 | //h4 | //div')
		raise 'no divs? wtf?' if nodes.length < 1
		nodes.each { |node|
			case node.name

			# Double check MSB number and triple-check bulletin severity
			when 'h1'
				title = node.text.strip
				title.gsub!("\u2013", "-")
				if title != msb[:title]
					$stderr.puts msb[:title].inspect
					$stderr.puts title.inspect
					raise 'Title mismatch!'
				end
				msbn,sev = split_title(title)
				raise 'MSB mismatch!' if msbn != msb[:msb]
				raise 'Severity mismatch!' if sev != msb[:severity]

			when 'h2'
				case node['class']

				# Double-check subtitle
				when "subtitle"
					txt = node.text.strip
					raise 'subtitle mismatch!' if msb[:subtitle] != txt

				when "extra"
					msb[:section] = node.text.strip.gsub(' Information', '')
					$stderr.puts "  Entering \"#{msb[:section]}\"" if $debug

				else
					$stderr.puts "Unknown h2 encountered: " + node.inspect

				end

			when 'h3'
				msb[:subsection] = node.text.strip
				$stderr.puts "    Entering: \"#{msb[:subsection]}\"" if $debug

			when 'h4'
				msb[:subsubsection] = node.text.strip
				$stderr.puts "      Entering: \"#{msb[:subsubsection]}\"" if $debug

			#when 'h5'
			#	msb[:subsubsubsection] = node.text.strip
			#	$stderr.puts "        Entering: \"#{msb[:subsubsubsection]}\"" if $debug

			when 'div'
				#$stderr.puts " DIV1: %-30s %s" % [ node['id'], node['class'] ] if $debug

				case node['class']
				when nil
					# Skip these div's with these ids
					next if [ nil, "msviMasthead",
						"msKiev",
						"msviLocalToolbar",
						"msviLocalFooter",
						"msviGlobalFooter"
					].include? node['id']

					scrape_div(node, msb)

				when 'expandoIndent'
					# ignore

				# Scrape the date!
				when "date"
					raise 'date already found!' if msb[:date]
					txt = node.text.strip
					dates = txt.split(' | ')
					dates.each { |d|
						parts = d.split(': ')
						case parts[0]
						when "Published"
							msb[:date] = Date.parse(parts[1])
							
						when "Updated"
							msb[:updated] = Date.parse(parts[1])
							
						else
							$stderr.puts "Unknown date type \"#{parts[0]}\" encountered"
							
						end
					}

				when "ancestorLinks", "dataTableBottomMargin", "msviLSBsform", "secTop"
					# ignore
				when"mnpAds", "mnpInherit", "mnpMenuBorder", "mnpMenuRow", "mnpMenuTop" 
					# ignore

				else
					$stderr.puts "Unknown div class #{node['class'].inspect} encountered"

				end

			else
				$stderr.puts "Unknown element encountered: " + node.inspect

			end
		}

		# Add it to the list ..
		ret << msb
		#break
	}
	ret
end


bulletins = get_bulletins()

#bulletins.sort! { |a,b| a[:date] <=> b[:date] }

bulletins.each { |msb|
	show_msb(msb)
}
