#!/bin/bash

date=`/bin/date "+%Y%m%d"`
ldate=`/bin/date "+%Y%m%d-%H%M%S"`
outdir='./malwareurls'

# You'll need to fix these paths, depending on your specific platform.
wget='/opt/local/bin/wget'

# If you have a BSD-based distro you don't have a -R switch to sort, so you need GNU's.
sort='/opt/local/libexec/gnubin/sort'

echo "Making sure directories exist..."
  if [ -d $outdir ]; then
        echo "All good."
	  else
	mkdir $outdir
	echo "Done."
  fi

echo ""
echo "Getting latest MalwareDomains list..."
  $wget "http://www.malwaredomains.com/updates/?C=M;O=D" -O $outdir/malwaredomains$date.html
  mdlatest=`cat $outdir/malwaredomains$date.html | grep ".txt"| head -1 | sed 's/^.*href="//' | sed 's/">.*//'`

  if [ -a $outdir/malwaredomains-$mdlatest ]; then
	echo "Latest already available."
	  else
	$wget "http://www.malwaredomains.com/updates/$mdlatest" -O $outdir/malwaredomains-$mdlatest 
  fi
echo "Done."

echo ""
echo "Getting latest MalwarePatrol list..."
  if [ -a $outdir/malwarepatrol$date.txt ]; then
	echo "Latest already available."
	  else
  $wget "http://www.malwarepatrol.net/cgi/submit?action=list" -O $outdir/malwarepatrol$date.txt
  fi
echo "Done."

echo ""
echo "Getting the latest BLADE list..."
  if [ -a $outdir/blade$date.csv ]; then
	echo "Latest already available."
	  else
  $wget http://www.blade-defender.org/eval-lab/blade.csv -O $outdir/blade$date.csv
  fi
echo "Done."

echo ""
echo "Getting the latest Pareto MDL..."
  if [ -a $outdir/mdl$date.xml ]; then
	echo "Latest already available."
	  else
  $wget  http://www.malwareblacklist.com/mdl.xml -O $outdir/mdl$date.xml
  fi
echo "Done."

echo ""
echo "Prepping the malware URL list:"

  if [ -a $outdir/malwareurls$date.txt ]; then
        echo "Backing up previous $date list..."
	mv $outdir/malwareurls$date.txt $outdir/malwareurls$ldate.txt
          else
	echo "No need to back up a previous list..."
  fi

  echo "Parsing and sorting..."

  grep -v "#" $outdir/malwarepatrol$date.txt > /tmp/malwareurlstmp$date.txt

  # MalwareDomains' BL only lists TLD, so we add both short name and full name to the list. 
  awk -F " " '{print $1}' $outdir/malwaredomains-$mdlatest | sed 's+^+http://+' >> /tmp/malwareurlstmp$date.txt
  awk -F " " '{print $1}' $outdir/malwaredomains-$mdlatest | sed 's+^+http://www.+' >> /tmp/malwareurlstmp$date.txt

  awk -F "," '{print $9}' $outdir/blade$date.csv | sed 's/"//g' | sed 's/hxxp/http/' >> /tmp/malwareurlstmp$date.txt

  # Pareto has some weird output in their XML, some odd UTF-8 and some improperly escaped HTML. 
  grep '<title>' $outdir/mdl$date.xml | sed 's+<title>+http://+;s+\ (.*++' | strings | sed 's+amp;++g' >> /tmp/malwareurlstmp$date.txt

  # Uncomment this and comment the line below if you don't want a random sort. We set locale to deal with UTF-8.
  # LC_ALL='C' $sort -u /tmp/malwareurlstmp$date.txt | sed '/^$/d' > $outdir/malwareurls$date.txt
  LC_ALL='C' $sort -u /tmp/malwareurlstmp$date.txt | LC_ALL='C' $sort -R | sed '/^$/d' > $outdir/malwareurls$date.txt

  echo "Done."

echo ""
urlcount=`wc -l $outdir/malwareurls$date.txt | awk -F " " '{print $1}'`

echo "Obtained $urlcount malware URLs. Enjoy."
